#
# Copyright (C) 2025 Indiscale GmbH <info@indiscale.com>
# Copyright (C) 2025 Florian Spreckelsen <f.spreckelsen@indiscale.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
import linkahead as db

from .utils import get_entity_name


def get_container_identifier(cont: db.Record) -> str:
    """Return an identifier string that uniquely identifies the given
    container entity in a registration/upload/export table file.

    Parameters
    ----------
    cont : db.Record
        The container record to be identified.

    Returns
    -------
    identifier : str
        The identifying string that uniquly identifies the given
        container.
    """
    if (cont.get_property(get_entity_name("container_label_prop")) is not None and
            cont.get_property(get_entity_name("container_label_prop")).value):
        return cont.get_property(get_entity_name("container_label_prop")).value
    elif cont.name:
        return cont.name
    return cont.id
