#
# Copyright (C) 2025 Indiscale GmbH <info@indiscale.com>
# Copyright (C) 2025 Florian Spreckelsen <f.spreckelsen@indiscale.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
import linkahead as db

from caosadvancedtools.datainconsistency import DataInconsistencyError

from .utils import get_entity_name

from linkahead.cached import cached_get_entity_by, cached_query as cquery

def get_person(text: str) -> db.Record:
    """Return the Person Record that is specifed as 'Main User' or
    'Sampling Person' in data.

    If the Person is not present in the database, an Exception is
    raised. Creating new Person Reconrds can only be done by a
    priviledged user.

    """
    # Check in which format the person is identified:
    person_identifier = text.split(", ")
    if len(person_identifier) == 1:
        person = _get_person_by_id(person_identifier[0])
    else:
        person = _get_person_by_fullname(person_identifier[1], person_identifier[0])

    return person


def _get_person_by_fullname(first_name: str, last_name: str) -> db.Record:
    # seach for person in db
    res = db.execute_query(
        f"FIND '{get_entity_name('Person')}' WITH "
        f"'{get_entity_name('first_name_prop')}'='{first_name}' "
        f"AND '{get_entity_name('last_name_prop')}'='{last_name}'"
    )
    # if person doesn't exist in database...
    if len(res) == 0:
        # There is not enought data in the template to create a new Person record. Hence, we have to raise an Exception
        error_msg = (
            f"There is no person Record with '{get_entity_name('first_name_prop')}' "
            f"= '{first_name}' "
            f"AND '{get_entity_name('last_name_prop')}' = '{last_name}' in the database. ")
        raise DataInconsistencyError(error_msg)
    else:
        return res[0]


def _get_person_by_id(ident: str) -> db.Record:
    options = cquery(f"FIND '{get_entity_name('Person')}' WITH ID='{ident}' ")
    if len(options)==1:
        return options[0]
    options = cquery(f"FIND '{get_entity_name('Person')}' WITH "
        f"'{get_entity_name('abbreviation_prop')}'='{ident}' ")
    if len(options) == 1:
        return options[0]
    elif len(options) == 0:
        raise DataInconsistencyError(
            f"Could not find a {get_entity_name('Person')} with ID {ident}."
        )
    else:
        raise DataInconsistencyError(
            f"Found multiple {get_entity_name('Person')} with {get_entity_name('abbreviation_prop')}={ident}."
        )
